package com.compdfkit.pdfviewer.customview.watermarkview;

import android.content.ClipData;
import android.content.ClipDescription;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Matrix;
import android.graphics.drawable.Drawable;
import android.os.Build;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.widget.ImageView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.content.ContextCompat;

import com.compdfkit.pdfviewer.R;
import com.compdfkit.pdfviewer.entity.WatermarkSetting;

public class ImageWatermarkView extends WatermarkView {
    private ImageView watermarkImage;

    private Bitmap initBitmap;

    private Bitmap resizeBitmap;

    public ImageWatermarkView(@NonNull Context context) {
        this(context, null);
    }

    public ImageWatermarkView(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public ImageWatermarkView(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        this(context, attrs, defStyleAttr, 0);
    }

    public ImageWatermarkView(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
        init(context);
    }

    private void init(Context context) {
        currentContext = context;
        inflater = (LayoutInflater) currentContext.getSystemService(Context.LAYOUT_INFLATER_SERVICE);

        currentView = inflater.inflate(R.layout.watermark_image_view, this, true);

        watermarkImage = currentView.findViewById(R.id.image_watermark);
        watermarkImage.post(() -> {
            Drawable drawable = ContextCompat.getDrawable(currentContext, R.drawable.pdf);
            assert drawable != null;
            initBitmap = Bitmap.createBitmap(drawable.getIntrinsicWidth(), drawable.getIntrinsicHeight(), Bitmap.Config.ARGB_4444);
            Canvas canvas = new Canvas(initBitmap);
            drawable.setBounds(0, 0, canvas.getWidth(), canvas.getHeight());
            drawable.draw(canvas);

            resizeBitmap = Bitmap.createBitmap(initBitmap, 0, 0, initBitmap.getWidth(), initBitmap.getHeight());
        });

        rollingHandle = currentView.findViewById(R.id.watermark_i_handle);

        currentView.setOnTouchListener((view, motionEvent) -> {
            switch (motionEvent.getAction()) {
                case MotionEvent.ACTION_DOWN:
                    break;
                case MotionEvent.ACTION_UP:
                    view.performClick();
                    break;
                default:
                    break;
            }
            ClipData.Item item = new ClipData.Item((CharSequence) view.getTag());
            ClipData dragData = new ClipData(
                    (CharSequence) view.getTag(),
                    new String[] { ClipDescription.MIMETYPE_TEXT_PLAIN },
                    item);
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                view.startDragAndDrop(dragData, new DragShadowBuilder(rollingHandle), null, 0);     //自定义DragShadowBuilder
            }

            currentEvent = 0;
            return true;
        });
        setRollingHandle();
    }

    @Override
    public void setRollingHandle() {
        rollingHandle.setOnTouchListener(new OnTouchListener() {
            @Override
            public boolean onTouch(View view, MotionEvent motionEvent) {
                ClipData.Item item = new ClipData.Item((CharSequence) view.getTag());
                ClipData dragData = new ClipData(
                        (CharSequence) view.getTag(),
                        new String[] { ClipDescription.MIMETYPE_TEXT_PLAIN },
                        item);
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                    view.startDragAndDrop(dragData, new DragShadowBuilder(rollingHandle), null, 0);     //自定义DragShadowBuilder
                }

                currentEvent = 1;
                return true;
            }
        });
    }

    @Override
    public void setWatermarkOpacity(float opacity) {
        watermarkImage.setAlpha(opacity);
    }

    @Override
    public void setWatermarkScale(float scale) {
        setScale(scale);
        Matrix matrix = new Matrix();
        matrix.postScale(scale, scale);
        resizeBitmap = Bitmap.createBitmap(initBitmap, 0, 0, initBitmap.getWidth(), initBitmap.getHeight(), matrix, true);

        setWatermarkImageContent(resizeBitmap);
    }

    @Override
    public WatermarkSetting completeSetting(WatermarkSetting setting) {
        Matrix matrix = new Matrix();
        matrix.postScale(1 / sizeConvertRate, 1 / sizeConvertRate);
        Bitmap resultBitmap = Bitmap.createBitmap(resizeBitmap, 0, 0, resizeBitmap.getWidth(), resizeBitmap.getHeight(), matrix, true);

        setting.setType(1);
        setting.setImage(resultBitmap);
        setting.setiWidth(resultBitmap.getWidth());
        setting.setiHeight(resultBitmap.getHeight());
        setting.setHorizOffset(getPDFXOffset());
        setting.setVertOffset(getPDFYOffset());
        setting.setRotation(rotationRadian);
        return setting;
    }

    @Override
    public void setSizeConvertRate(float sizeConvertRate) {
        super.setSizeConvertRate(sizeConvertRate);
    }

    @Override
    public Bitmap getFullScreenBitmap(int width, int height, int hSpace, int vSpace, int xOffset, int yOffset, float rotation) {
        Matrix matrix = new Matrix();
        matrix.setScale(getScale(), getScale());

        Bitmap tempBitmap = Bitmap.createBitmap(initBitmap, 0, 0, initBitmap.getWidth(), initBitmap.getHeight(), matrix, true);
        return generateFullScreenBitmap(tempBitmap, width, height, hSpace, vSpace, xOffset, yOffset, rotation);
    }

    public void setWatermarkImageContent(Bitmap content) {
        watermarkImage.setImageBitmap(content);
    }

    public void setWatermarkImageFromPath(String imagePath) {
        if (imagePath != null) {
            initBitmap = BitmapFactory.decodeFile(imagePath);
            setWatermarkScale(getScale());
        } else {
            Toast.makeText(currentContext, "获取图片失败", Toast.LENGTH_SHORT).show();
        }
    }
}
