package com.compdfkit.pdfviewer.customview.watermarkview;

import android.content.ClipData;
import android.content.ClipDescription;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.os.Build;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.AttributeSet;
import android.util.Log;
import android.util.TypedValue;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.widget.AppCompatEditText;
import androidx.constraintlayout.widget.ConstraintLayout;

import com.compdfkit.pdfviewer.R;
import com.compdfkit.pdfviewer.entity.WatermarkSetting;
import com.google.android.material.dialog.MaterialAlertDialogBuilder;

public class TextWatermarkView extends WatermarkView{
    private TextView watermarkText;

    private float textDefaultSize = 123;

    private static final float verticalOffsetCompensate = 2.6966689f;

    public TextWatermarkView(@NonNull Context context) {
        this(context, null);
    }

    public TextWatermarkView(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public TextWatermarkView(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        this(context, attrs, defStyleAttr, 0);
    }

    public TextWatermarkView(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
        init(context);
    }

    private void init(Context context) {
        currentContext = context;
        inflater = (LayoutInflater) currentContext.getSystemService(Context.LAYOUT_INFLATER_SERVICE);

        currentView = inflater.inflate(R.layout.watermark_text_view, this, true);

        watermarkText = currentView.findViewById(R.id.text_watermark);
        rollingHandle = currentView.findViewById(R.id.watermark_t_handle);

        setRollingHandle();

        this.setOnTouchListener(new OnTouchListener() {
            float x, y;
            @Override
            public boolean onTouch(View view, MotionEvent motionEvent) {
                switch (motionEvent.getAction()) {
                    case MotionEvent.ACTION_DOWN:
                        x = motionEvent.getRawX();
                        y = motionEvent.getRawY();
                        break;
                    case MotionEvent.ACTION_MOVE:
                        if (Math.abs(motionEvent.getRawX() - x) > 6 && Math.abs(motionEvent.getRawY() - y) > 6) {
                            ClipData.Item item = new ClipData.Item((CharSequence) view.getTag());
                            ClipData dragData = new ClipData(
                                    (CharSequence) view.getTag(),
                                    new String[] { ClipDescription.MIMETYPE_TEXT_PLAIN },
                                    item);
                            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                                view.startDragAndDrop(dragData, new DragShadowBuilder(rollingHandle), null, 0);     //自定义DragShadowBuilder
                            }
                            currentEvent = 0;
                        }
                        return true;
                    case MotionEvent.ACTION_UP:
                        view.performClick();
                        break;
                    default:
                        break;
                }
                return true;
            }
        });
    }

    @Override
    public boolean performClick() {
        LayoutInflater inflater = LayoutInflater.from(currentContext);
        final ConstraintLayout textEditorLayout = (ConstraintLayout) inflater.inflate(R.layout.text_editor, null);
        final AppCompatEditText editText = textEditorLayout.findViewById(R.id.text_editor);

        final Button[] positiveButton = new Button[1];

        editText.setText(watermarkText.getText());
        MaterialAlertDialogBuilder inputDialog = new MaterialAlertDialogBuilder(currentContext);
        inputDialog.setTitle(R.string.text_watermark).setView(textEditorLayout);
        inputDialog.setPositiveButton(R.string.done, (dialogInterface, i) -> setWatermarkTextContent(editText.getText().toString()));

        editText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {}

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {}

            @Override
            public void afterTextChanged(Editable editable) {
                positiveButton[0].setEnabled(!editText.getText().toString().equals(""));
            }
        });

        inputDialog.setNegativeButton(R.string.cancel, (dialogInterface, i) -> {});

        AlertDialog dialog = inputDialog.create();
        dialog.show();

        positiveButton[0] = dialog.getButton(AlertDialog.BUTTON_POSITIVE);
        return super.performClick();
    }

    @Override
    public void setRollingHandle() {
        rollingHandle.setOnTouchListener((View view, MotionEvent motionEvent) -> {
            currentEvent = 1;
            ClipData.Item item = new ClipData.Item((CharSequence) view.getTag());
            ClipData dragData = new ClipData(
                    (CharSequence) view.getTag(),
                    new String[] { ClipDescription.MIMETYPE_TEXT_PLAIN },
                    item);
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                view.startDragAndDrop(dragData, new DragShadowBuilder(rollingHandle), null, 0);     //自定义DragShadowBuilder
            }
            return true;
        });
    }

    @Override
    public void setWatermarkOpacity(float opacity) {
        watermarkText.setAlpha(opacity);
    }

    @Override
    public void setWatermarkScale(float scale) {
        setScale(scale);
        watermarkText.setTextSize(TypedValue.COMPLEX_UNIT_PX, textDefaultSize * scale);
    }

    @Override
    public void setTextWatermarkColor(int color) {
        watermarkText.setTextColor(color);
    }

    @Override
    public WatermarkSetting completeSetting(WatermarkSetting setting) {
        setting.setType(0);
        setting.setHorizOffset(getPDFXOffset());
        setting.setVertOffset(getPDFYOffset());
        setting.setText(watermarkText.getText().toString());
        setting.setFontSize(textDefaultSize / sizeConvertRate * scale);
        setting.setTextRGBColor(watermarkText.getCurrentTextColor());
        setting.setRotation(rotationRadian);
        return setting;
    }

    @Override
    public void setSizeConvertRate(float sizeConvertRate) {
        super.setSizeConvertRate(sizeConvertRate);
        setTextDefaultSize(sizeConvertRate * 62);
    }

    @Override
    public Bitmap getFullScreenBitmap(int width, int height, int hSpace, int vSpace, int xOffset, int yOffset, float rotation) {
        Paint paint = new Paint();
        paint.setTextSize(textDefaultSize);
        paint.setTextAlign(Paint.Align.LEFT);
        paint.setColor(watermarkText.getCurrentTextColor());

        Paint.FontMetricsInt fm = paint.getFontMetricsInt();

        int textwidth = (int)paint.measureText((String) watermarkText.getText());
        int textheight = fm.descent - fm.ascent;

        Bitmap bitmap = Bitmap.createBitmap(textwidth, textheight, Bitmap.Config.ARGB_8888);
        Canvas canvas = new Canvas(bitmap);
        canvas.drawText((String) watermarkText.getText(), 0, fm.leading - fm.ascent + 5, paint);
        canvas.save();

        Matrix matrix = new Matrix();
        matrix.setScale(getScale(), getScale());
        bitmap = Bitmap.createBitmap(bitmap, 0, 0, bitmap.getWidth(), bitmap.getHeight(), matrix, true);

        return generateFullScreenBitmap(bitmap, width, height, hSpace, vSpace, xOffset, yOffset, rotation);
    }

    public void setWatermarkTextContent(String content) {
        watermarkText.setText(content);
    }

    public void setWatermarkTextFontSize(float textFontSize) {
        watermarkText.setTextSize(TypedValue.COMPLEX_UNIT_PX, textFontSize * sizeConvertRate);
    }

    public void setTextDefaultSize(float textDefaultSize) {
        this.textDefaultSize = textDefaultSize;
        watermarkText.setTextSize(TypedValue.COMPLEX_UNIT_PX, textDefaultSize);
        Log.d("Watermark attribute", "Text Watermark: " + watermarkText.getTextSize());
    }
}
